#region Copyright Syncfusion Inc. 2001 - 2007
//
//  Copyright Syncfusion Inc. 2001 - 2007. All rights reserved.
//
//  Use of this code is subject to the terms of our license.
//  A copy of the current license can be obtained at any time by e-mailing
//  licensing@syncfusion.com. Any infringement will be prosecuted under
//  applicable laws. 
//
#endregion

using System;
using System.Drawing;
using System.Collections;
using System.ComponentModel;
using System.Runtime.Serialization;
using System.Windows.Forms;
using System.IO;

using Syncfusion.Windows.Forms;
using Syncfusion.Windows.Forms.Grid;


namespace GridSample
{
    public class CalendarCellModel : GridGenericControlCellModel
    {
        public CalendarCellModel(GridModel grid)
            : base(grid)
        {
        }

        protected CalendarCellModel(SerializationInfo info, StreamingContext context)
            : base(info, context)
        {
        }

        public override GridCellRendererBase CreateRenderer(GridControlBase control)
        {
            return new CalendarCellRenderer(control, this);
        }

        static Size calendarControlSize = Size.Empty;

        public static Size CalendarControlSize
        {
            get
            {
                if (calendarControlSize.IsEmpty)
                {
                    // Determine default size of MonthCalendar when shown in a form
                    TopLevelWindow f = new TopLevelWindow();
                    f.Location = new Point(10000, 10000);
                    f.Size = new Size(500, 500);
                    f.ShowWindowTopMost();
                    MonthCalendar drawCalendar = new MonthCalendar();
                    f.Controls.Add(drawCalendar);
                    f.Visible = true;
                    calendarControlSize = drawCalendar.Size;
                    f.Dispose();
                }
                return calendarControlSize;
            }
        }

        protected override Size OnQueryPrefferedClientSize(Graphics g, int rowIndex, int colIndex, GridStyleInfo style, GridQueryBounds queryBounds)
        {
            Size size = CalendarControlSize;
            size.Width += 8;
            size.Height += 8;
            return size;
        }

        public static void InitializeCalendar(MonthCalendar c, GridStyleInfo style)
        {
            InitializeCalendar(c, style.CellValue);
        }

        public static void InitializeCalendar(MonthCalendar c, object controlValue)
        {
            if (controlValue is DateTime)
                c.SetDate((DateTime)controlValue);
            else if (controlValue is string && ((string)controlValue).Length > 0)
                c.SetDate(DateTime.Parse((string)controlValue));
            else
                c.SetDate(DateTime.Today);
        }
    }

	public class CalendarCellRenderer: GridGenericControlCellRenderer
	{
		MonthCalendar m_editCalendar;
        
		public CalendarCellRenderer(GridControlBase grid, GridCellModelBase cellModel)
			: base(grid, cellModel)
		{
			SupportsFocusControl = true;
		}

		protected override void Dispose(bool disposing)
		{
			if (disposing)
			{
                if (m_editCalendar != null)
                {
                    m_editCalendar.DateChanged -= new DateRangeEventHandler(calendar_DateChanged);
                    // Do not dispose m_editCalendar. Style takes care of this.
                    m_editCalendar = null;
                }
			}

			base.Dispose (disposing);
		}

		protected override bool OnSaveChanges()
		{
			this.Grid.Model[RowIndex, ColIndex].CellValue = m_editCalendar.SelectionStart;
			return true;
		}

        protected override void OnActivated()
		{
			// Works around some interaction problem with PropertyGrid specific to this sample: force redrawing of
			// calendar after property grid was refreshed.
			Timer t = new Timer();
			t.Tick += new EventHandler(t_Tick);
			t.Interval = 200;
			t.Start();

			base.OnActivated ();
		}

		private void t_Tick(object sender, EventArgs e)
		{
			Timer t = (Timer) sender;
			t.Tick += new EventHandler(t_Tick);
			t.Dispose();

            if (m_editCalendar != null)
			    m_editCalendar.Refresh();
		}


        /// <summary>
        /// Active calendar for editing current cell
        /// </summary>
        MonthCalendar EditCalendar
        {
            get
            { 
                return m_editCalendar; 
            }
            set
            {
                MonthCalendar drawCalendar = value;
                if (!Object.ReferenceEquals(m_editCalendar, drawCalendar))
                {
                    if (m_editCalendar != null)
                        m_editCalendar.DateChanged -= new DateRangeEventHandler(calendar_DateChanged);

                    m_editCalendar = drawCalendar;
                    SetControl(m_editCalendar);

                    if (m_editCalendar != null)
                        m_editCalendar.DateChanged += new DateRangeEventHandler(calendar_DateChanged);
                }
            }
        }
		
		protected override void OnDraw(Graphics g, Rectangle clientRectangle, int rowIndex, int colIndex, GridStyleInfo style)
		{
			clientRectangle.Size = GridUtil.Max(clientRectangle.Size, CalendarCellModel.CalendarControlSize);

            MonthCalendar drawCalendar = style.Control as MonthCalendar;

            if (drawCalendar != null)
            {
                if (this.ShouldDrawFocused(rowIndex, colIndex))
                {
                    EditCalendar = drawCalendar;
                }
                else
                    CalendarCellModel.InitializeCalendar(drawCalendar, style);
            }

			base.OnDraw (g, clientRectangle, rowIndex, colIndex, style);
		}

		protected override void InitializeControlText(object controlValue)
		{
            MonthCalendar drawCalendar = StyleInfo.Control as MonthCalendar;

            if (drawCalendar != null)
            {
                EditCalendar = drawCalendar;
                CalendarCellModel.InitializeCalendar(this.m_editCalendar, controlValue);
            }
		}

        protected override void OnHasFocusControlChanged()
        {
            if (!this.HasFocusControl)
            {
                if (m_editCalendar != null)
                {
                    Grid.SuspendLayout();
                    m_editCalendar.Location = new Point(10000, 10000);
                    Grid.ResumeLayout(false);
                }
            }
            base.OnHasFocusControlChanged();
        }

		protected override Rectangle OnLayout(int rowIndex, int colIndex, GridStyleInfo style, Rectangle innerBounds, Rectangle[] buttonsBounds)
		{
			Rectangle r = base.OnLayout (rowIndex, colIndex, style, innerBounds, buttonsBounds);

			r.Inflate(-1, -1);
			
			// center windows forms calendar inside cell.
			r = GridUtil.CenterInRect(r, CalendarCellModel.CalendarControlSize);
			r.Size = GridUtil.Max(r.Size, CalendarCellModel.CalendarControlSize);
			return r;
		}

		private void calendar_DateChanged(object sender, DateRangeEventArgs e)
		{
			// Raise CurrentCellChanging event and set CurrentCell.Modified = true
			if (base.NotifyCurrentCellChanging())
			{
				ControlValue = m_editCalendar.SelectionStart;
				base.NotifyCurrentCellChanged();
			}
		}

        // We chose to provide "Calendar" controls in MainForm.cs:
        // The benefit of this is the controls will always be sticking with the cell
        // when the user inserts/remove or moves cells.
        /*
         * 		for (int row = 2; row <= 6; row++)
			{
				if (row % 2 == 0)
				{
					style = gridControl1[row, 2];
					style.CellType = "Calendar";
                    style.Control = new MonthCalendar();
					style = gridControl1[row, 3];
					style.CellType = "Calendar";
                    style.Control = new MonthCalendar();
				}
			}
         */
	}

}
